      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     GRAVITY TIDE CALCULATION
C
      DIMENSION   GDW(8784)
      DIMENSION   FACT(25),  PHASE(25)
C
C
C     TABLE OF TIDAL FACTOR AND PHASE
C       M0+S0,   LONG PERIOD,
C          Q1,       O1,       M1,   { ( PI1,      P1,)      S1,
C        ( K1,       PSI1,     PHI1,) }  J1,       OO1,
C          2N2,      N2,     ( ALPHA2,   M2,       BETA2,)
C        ( LAMBDA2,  L2,)  { ( T2,       S2,)      K2,}
C          M3,       M4
C
C    AMPLITUDE FACTOR
C     NOTE : FACTOR OF CONSTANT TERM (M0S0) IS 1.00, NOT 1.16.
      DATA  FACT
     *  / 1.0000D0, 1.1550D0,
     *    1.1582D0, 1.1580D0, 1.1574D0, 1.1544D0, 1.1529D0, 1.1497D0,
     *    1.1386D0, 1.2329D0, 1.1718D0, 1.1599D0, 1.1592D0,
     *    1.1600D0, 1.1600D0, 1.1600D0, 1.1600D0, 1.1600D0,
     *    1.1600D0, 1.1600D0, 1.1600D0, 1.1600D0, 1.1600D0,
     *    1.0700D0, 1.0365D0  /
C
C    PHASE (LOCAL, LAG:NEGATIVE, DEGREE)
      DATA  PHASE
     *  / 0.00D0,   0.00D0,
     *    0.00D0,   0.00D0,   0.00D0,   0.00D0,   0.00D0,   0.01D0,
     *    0.06D0,   0.78D0,   0.03D0,   0.00D0,   0.00D0,
     *    0.00D0,   0.00D0,   0.00D0,   0.00D0,   0.00D0,
     *    0.00D0,   0.00D0,   0.00D0,   0.00D0,   0.00D0,
     *    0.00D0,   0.00D0    /
C
C
C     ESASHI OBSERVED FACTORS AND PHASES
C     BELOW FACTORS AND PHASES ARE AFFECTED BY OCAEN TIDE LOAD.     
C
CC    DATA  FACT
CC   *  / 1.0000D0, 1.1550D0,
CC   *    1.2202D0, 1.2192D0, 1.2148D0, 1.2076D0, 1.2112D0, 1.3727D0,
CC   *    1.1992D0, 1.2778D0, 1.2285D0, 1.2082D0, 1.2034D0,
CC   *    1.1944D0, 1.1783D0, 1.1905D0, 1.1905D0, 1.1905D0,
CC   *    1.1656D0, 1.2032D0, 1.2290D0, 1.2161D0, 1.2139D0,
CC   *    1.0940D0, 1.0365D0  /
C
C     PHASE (LOCAL, LAG:NEGATIVE, DEGREE)
CC    DATA  PHASE
CC   *  / 0.00D0,   0.00D0,
CC   *    1.85D0,   1.18D0,   0.31D0,   0.30D0,   0.18D0,  10.23D0,
CC   *    0.15D0,  -2.00D0,  -0.02D0,  -0.37D0,  -1.45D0,
CC   *    0.38D0,   0.50D0,   1.34D0,   1.34D0,   1.34D0,
CC   *    1.74D0,   1.71D0,  -0.14D0,   0.14D0,   0.64D0,
CC   *    1.89D0,   0.00D0    / 
C
C
C    LOCATION
C
      AEAST  =  141.335D0
      ANORTH =   39.148D0
      HI     =  393.0D0
C
C    FIRST EPOCH
C
      IYEAR  = 1999
      IMONTH =    1
      IDAY   =    1
      IHOUR  =    0
      IMIN   =    0
      SEC    =    0.0D0
C
C    TIME SYSTEM
C
      ETMUT  =   63.5D0
      TIMSYS =    0.0D0
C
C    NUMBER OF CALCULATION, TIME STEP IN HOURS
C
      NUM    =  744
      STEP   =    1.0D0
C
C     MINIMUM AMPLITUDE TO BE ADOPTED IN THE CALCULATION
C     LARGER 'SLIMIT' SAVES COMPUTATION TIME LOSING PRECISION.
C     0.1 MICRO GAL PRECISION (SLIMIT=45.D-6) MIGHT BE ENOUGH
C     FOR TIDAL CORRECTIONS.
C        SLIMIT, NUMBER OF WAVES, SD, MAXIMUM DIFFERENCE
C         0.D-6,      1200,        -           -
C        20.D-6,       879,      0.012,      0.05 MICRO GAL
C        45.D-6,       597,      0.022,      0.10
C       100.D-6,       404,      0.051,      0.22
C       200.D-6,       288,      0.089,      0.35
C
      SLIMIT =  0.0D0
CC    SLIMIT = 45.0D-6
C
C   < INITIALIZATION >
C     SUBROUTINE GTIDE1 MUST BE CALLED BEFORE THE EXECUTION OF
C     GTIDE2.  THIS ROUTINE MAY BE CALLED ONLY ONE TIME IF THE
C     LOCATION AND TIDAL FACTORS ARE NOT CHANGED.
C
      CALL GTIDE1( AEAST, ANORTH, HI, SLIMIT, FACT, PHASE )
C
C   < THEORETICAL TIDE CALCULATION >
C     THIS ROUTINE IS DESIGNED TO CALUCULATE EQUI-TIME INTERVAL.
C     FOR THE CALCULATION OF RANDOM TIME INTERVAL, SET NUM=1 AND
C     STEP=any value, OR MODIFY THIS ROUTINE A LITTLE.
C
      CALL GTIDE2( GDW, NUM, STEP,
     *             IYEAR, IMONTH, IDAY, IHOUR, IMIN, SEC,
     *             ETMUT, TIMSYS )
C
C
C     UPWARD ATRACTION (DECREASE OF GRAVITY) IS POSITIVE.
      WRITE(6,150) (-GDW(I),I=1,NUM)
  150 FORMAT(6F10.2)
C
      STOP
      END
      SUBROUTINE GTIDE1( RM, PH, HI, SLIMIT, FCT, PHDIF )
C
C     INITIALIZATION ROUTINE
C
C     CODED BY Y. TAMURA    ( NOVEMBER 21, 1996 )
C       NATIONAL ASTRONOMICAL OBSERVATORY, MIZUSAWA
C       2-12 HOSHIGAOKA MIZUSAWA IWATE 023  JAPAN
C       TEL. (81)-197-227131   FAX (81)-197-223410
C       E-mail tamura@miz.nao.ac.jp
C
C     INPUT DATA
C       RM      : STATION PLACE   EAST LONGITUDE IN DEGREES
C       PH      :     "           NORTH LATITUDE     "
C       HI      :     "           HEIGHT         IN METERS
C       SLIMIT  : LOWER AMPLITUDE LIMIT TO BE CONTAINED, NORMALLY 0.D0
C       FCT     : OBSERVED AMPLITUDE FACTORS
C       PHDIF   : OBSERVED PHASE DIFFERENCES (LAG:NEGATIVE, IN DEGREES)
C
C     REFERENCES
C       TAMURA, Y. 1987, A HARMONIC DEVELOPMENT OF THE TIDE-GENERATING
C         POTENTIAL, MAREES TERRESTRES BULLETIN D'INFORMATIONS, 99,
C         PP6813-6855.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MAXC = 1200 )
C
      DIMENSION  FCT(25), PHDIF(25)
C
      DIMENSION         ZZ(4,0:4)
      DOUBLE PRECISION  K2, K3, K4
      DIMENSION         KUGIRI(25)
      REAL              ALIMIT
C
      REAL    DOOD, AMPL, AMPL2
      COMMON /TABLE4/ DOOD(8,MAXC), AMPL(MAXC), AMPL2(MAXC), IORD(MAXC)
      COMMON /TABLE5/ FACTX(MAXC), PHASEI(MAXC), ELON, MAXWV, NCALLD
C
C
      DATA  PI    / 3.14159 26535 89793 D0/
C
C     RE    : EARTH'S EQUATORIAL RADIUS IN CM
C     FLAT  : EARTH'S FLATTENING
C     GE    : GEOCENTRIC GRAVITATIONAL CONSTANT
C     FMU   : MASS RATIO OF MOON AND EARTH
C     SINPI : MOON'S SIN PARALLAX
      DATA  RE    / 6378137.D2 /
      DATA  FLAT  / 3.35292D-3 /
      DATA  GE    / 3.98600448D20 /
      DATA  FMU   / 0.012300034D0 /
      DATA  SINPI / 3422.448D0 /
C
C     LOVE NUMBERS
      DATA  K2, K3, K4  / 0.299D0,  0.093D0,  0.042D0  /
      DATA  H2, H3, H4  / 0.606D0,  0.291D0,  0.178D0  /
C
C     TIDE GROUP
      DATA  KUGIRI
     *     /    2, 281,
     *        424, 482, 530, 537, 547, 551, 569, 573, 586,  626, 731,
     *        830, 880, 893, 912, 936, 944, 962, 982, 991, 1108,
     *       1190, 1200  /
C
C
C     CHECKING PARAMETERS
C
C
      ILL = 0
      IF( DABS(RM).GT.360.D0 )              ILL = 1
      IF( DABS(PH).GT. 90.D0 )              ILL = 1
      IF( DABS(HI).GT.9999.D0 )             ILL = 1
      IF( SLIMIT.GE.0.910000 )              ILL = 1
      IF( ILL .NE. 0 )  THEN
         WRITE(6,110)
         WRITE(6,120)  RM, PH, HI
         WRITE(6,130)  SLIMIT
         STOP
      END IF
  110 FORMAT(/' INVARID PARAMETERS      STOPPED IN GTIDE1')
  120 FORMAT( ' RM, PH, HI =',2F11.4,F10.2)
  130 FORMAT( ' SLIMIT     =',I8,F10.6)
C
C
C     READING HARMONIC TABLE
C
C
CC    OPEN ( 95, FILE = '/users/tamura/tide/SORTED.DAT',
CC   *       STATUS = 'OLD' )
      OPEN ( 95, FILE = '/home/tamura/tide/SORTED.DAT',
     *       STATUS = 'OLD' )
C
      DO 200 I = 1, MAXC
  200 READ(95,220)  IORD(I), (DOOD(J,I),J=1,8), AMPL(I), AMPL2(I)
  220 FORMAT( 7X, I1, 1X, 8F3.0, 15X, 2F11.6 )
C
      CLOSE ( 95 )
C
C
C     GEOGRAPHICAL CONSTANTS
C
C
      RAD = PI/180.D0
      CV  = 3.D0 / 4.D0 * FMU * GE * ( SINPI/3600.D0*RAD )**3 / RE**2
      PHI  = PH * RAD
      ALTI = HI * 100.D0
C
      COSPHI = DCOS( PHI )
      SINPHI = DSIN( PHI )
      E2 = 2.D0*FLAT - FLAT**2
      ROE = (   1.D0      / DSQRT( 1.D0-E2*SINPHI**2 ) + ALTI/RE )
     *      * COSPHI
      ROP = ( ( 1.D0-E2 ) / DSQRT( 1.D0-E2*SINPHI**2 ) + ALTI/RE )
     *      * SINPHI
      ROE2 = ROE**2
      ROP2 = ROP**2
C
C
C     GRAVITY IN MICRO GAL
      CV6  =  -1.0D6*CV
      ZZ(2,0) = CV6 * ( ROE*COSPHI - 2.D0*ROP*SINPHI )
      ZZ(2,1) = 2.D0 * CV6 * ( ROP*COSPHI + ROE*SINPHI )
      ZZ(2,2) = 2.D0 * CV6 * ROE * COSPHI
      ZZ(3,0) = 3.D0*DSQRT(5.D0)/2.D0 * CV6 * ( 2.D0*ROE*ROP
     *       * COSPHI + (ROE2-2.D0*ROP2)*SINPHI )
      ZZ(3,1) = 3.D0*DSQRT(15.D0)/16.D0 * CV6 * ( (3.D0*ROE2
     *         -4.D0*ROP2)*COSPHI - 8.D0*ROE*ROP*SINPHI )
      ZZ(3,2) = 3.D0*DSQRT(3.D0)/2.D0 * CV6 * ROE
     *           * ( 2.D0*ROP*COSPHI + ROE*SINPHI )
      ZZ(3,3) = 3.D0 * CV6 * ROE2 * COSPHI
      ZZ(4,0) = 0.5D0 * CV6
     *       * (  3.D0*ROE*( ROE2-4.D0*ROP2 )     *COSPHI
     *          + 4.D0*ROP*( 2.D0*ROP2-3.D0*ROE2 )*SINPHI )
      ZZ(4,1) = 448.D0 / ( 3.D0+DSQRT(393.D0))
     *          / DSQRT( 390.D0+2.D0*DSQRT(393.D0) ) * CV6
     *       * (  ROP*( 9.D0*ROE2-4.D0*ROP2 )*COSPHI
     *          + 3.D0*ROE*( ROE2-4.D0*ROP2 )*SINPHI )
      ZZ(4,2) = 28.D0/9.D0 * CV6
     *       * (  ROE*( ROE2-3.D0*ROP2 )*COSPHI
     *          - 3.D0*ROE2*ROP         *SINPHI )
      ZZ(4,3) = 16.D0*DSQRT(3.D0)/9.D0 * CV6
     *       * ( 3.D0*ROE2*ROP*COSPHI + ROE**3*SINPHI )
      ZZ(4,4) = 4.D0 * CV6 * ROE**3 * COSPHI
C
      DFACT2 = 1.D0 +           H2 -    1.50D0*K2
      DFACT3 = 1.D0 + 2.D0/3.D0*H3 - 4.D0/3.D0*K3
      DFACT4 = 1.D0 +     0.5D0*H4 -    1.25D0*K4
C
      RATIOA  = DFACT3/DFACT2
      RATIOB  = DFACT4/DFACT2
      RATIOC  = DFACT4/DFACT3
      ZZ(3,0) = ZZ(3,0)*RATIOA
      ZZ(3,1) = ZZ(3,1)*RATIOA
      ZZ(3,2) = ZZ(3,2)*RATIOA
      ZZ(4,0) = ZZ(4,0)*RATIOB
      ZZ(4,1) = ZZ(4,1)*RATIOB
      ZZ(4,2) = ZZ(4,2)*RATIOB
      ZZ(4,3) = ZZ(4,3)*RATIOC
C
C
C     FACTOR AND PHASE
C
C
      LL = 0
      KK = 1
      DO 500 I = 1, MAXC
         KO     = IORD(I)
         ALIMIT = SLIMIT
         IF( KO.EQ.3 )  ALIMIT = ALIMIT*0.67
         IF( KO.EQ.4 )  ALIMIT = ALIMIT*0.50
         IF( ABS(AMPL(I)) .LT. ALIMIT )  GO TO 400
C
         LL = LL + 1
C
         KS = INT( DOOD(1,I) )
         IF( MOD(KO+KS,2).EQ.0 )  THEN
            PHASE0 =  0.D0
         ELSE
            PHASE0 = -PI/2.D0
         END IF
C
         FACTX (LL) = ZZ(KO,KS) * FCT(KK)
         PHASEI(LL) = PHASE0 + PHDIF(KK)*RAD
         DO 300 J = 1, 8
  300    DOOD(J,LL) = DOOD(J,I)
         AMPL (LL)  = AMPL (I)
         AMPL2(LL)  = AMPL2(I)
CC       IORD (LL)  = IORD (I)
C
  400    IF( I.EQ.KUGIRI(KK) )  KK = KK + 1
  500 CONTINUE
C
C
C     END OF INITIALIZATION
C
C
      ELON   = RM
      MAXWV  = LL
      NCALLD = 123456789
C
CC    WRITE(6,*)  ' MAXWV =', MAXWV
C
      RETURN
      END
      SUBROUTINE GTIDE2( XT, NUMB, STEP,
     *                   JA, JB, JC, JD, JE, SEC, ETMUT, TIMSYS )

C     THEORETICAL TIDE CALCULATION WITH TAMURA'S TABLE
C
C     CODED BY Y. TAMURA    ( NOVEMBER 21, 1996 )
C       NATIONAL ASTRONOMICAL OBSERVATORY, MIZUSAWA
C       2-12 HOSHIGAOKA MIZUSAWA IWATE 023  JAPAN
C       TEL. (81)-197-227131   FAX (81)-197-223410
C       E-mail tamura@miz.nao.ac.jp
C
C     OUTPUT DATA
C       XT(I)   : GRAVITY TIDE ( UNIT IN MICRO GALS )
C               : DOWNWARD ATRACTION (I.E. INCREASE OF GRAVITY) 
C               : IS TAKEN TO BE POSITIVE.
C     INPUT DATA
C       NUMB    : NUMBER OF CALCULATIONS
C       STEP    : TIME INTERVAL IN HOURS
C       JA      : ORIGIN TIME  YEAR ( 1901 =< JA =< 2099 )
C       JB      :     "        MONTH
C       JC      :     "        DAY
C       JD      :     "        HOUR
C       JE      :     "        MINUTE
C       SEC     :     "        SECOND
C       ETMUT   : TD(ET) - UT  IN SECONDS
C       TIMSYS  : TIME CORRECTON ( JST: TIMSYS=-9.D0, UT: TIMSYS=0.D0 )
C
C     REFERENCES
C       TAMURA, Y. 1987, A HARMONIC DEVELOPMENT OF THE TIDE-GENERATING
C         POTENTIAL, MAREES TERRESTRES BULLETIN D'INFORMATIONS, 99,
C         PP6813-6855.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MAXC = 1200 )
C
      DIMENSION  XT(*)
C
C
      REAL    DOOD, AMPL, AMPL2
      COMMON /TABLE4/ DOOD(8,MAXC), AMPL(MAXC), AMPL2(MAXC), IORD(MAXC)
      COMMON /TABLE5/ FACTX(MAXC), PHASEI(MAXC), ELON, MAXWV, NCALLD
C
      DATA  PI  / 3.14159 26535 89793 D0/
C
C
C     INITIALIZED ?
C
C
      IF( NCALLD .NE. 123456789 )  THEN
         WRITE(6,200)
         STOP
      END IF
  200 FORMAT(/' ERROR     SUBROUTINE GTIDE1 IS NOT CALLED')
C
C
C     CHECKING PARAMETERS
C
C
      ILL = 0
      IF( NUMB.LE.0 )                          ILL = 1
      IF( JA.LE.1900 .OR. JA.GE.2100 )         ILL = 1
      IF( JB.LE.0 .OR. JB.GE.13 )              ILL = 1
      IF( JC.LE.0 .OR. JC.GE.32 )              ILL = 1
      IF( JD.LT.0 .OR. JD.GE.24 )              ILL = 1
      IF( JE.LT.0 .OR. JE.GE.60 )              ILL = 1
      IF( SEC  .LT.0.D0 .OR. SEC  .GE.60.D0 )  ILL = 1
      IF( ETMUT.LT.0.D0 .OR. ETMUT.GT.99.D0 )  ILL = 1
      IF( DABS(TIMSYS).GT.24.D0 )              ILL = 1
      IF( ILL.NE.0 )  THEN
         WRITE(6,210)
         WRITE(6,220)  NUMB, JA, JB, JC, JD, JE, SEC
         WRITE(6,230)  ETMUT, TIMSYS
         STOP
      END IF
  210 FORMAT(/' INVARID PARAMETERS      STOPPED IN GTIDE2')
  220 FORMAT( ' NUMB, JA, JB, JC  =',4I10 /
     *        '       ID, IE, SEC =',2I10,F10.2)
  230 FORMAT( ' TITMUT, TIMSYS    =',2F10.1)
C
C
C     UNIVERSAL TIME AND DYNAMICAL TIME AT THE FIRST EPOCH
C
C
      TU0 = ( TJULID( JA, JB, JC, JD, JE ) - 2451545.D0 + TIMSYS/24.D0 )
     *         / 36525.D0
      TD0 = TU0 + ETMUT/86400.D0/36525.D0
      DELT = STEP / 24.D0 / 36525.D0
      UT0  = DBLE( JD ) + DBLE( JE )/60.D0 + TIMSYS
C
C
C     LOOP OF TIME 
C
C
      RAD = PI/180.D0
C
      DO 2000 K = 1, NUMB
C
         TU   = TU0 + DELT*DBLE(K-1)
         TU2  = TU*TU
         TD   = TD0 + DELT*DBLE(K-1)
         TD2  = TD*TD
         UT   = UT0 + STEP*DBLE(K-1)
C
         EL0 = 280.4606184D0 + 36000.7700536D0*TU+ 0.00038793D0*TU2
         EL2 = 218.316656D0 + 481267.881342D0*TD - 0.001330D0 *TD2
     *         + 0.0040D0*DCOS( ( 29.D0+133.D0*TD)*RAD )
         EL1 =  15.D0*UT + ELON + EL0 - EL2
         EL3 = 280.466449D0  + 36000.769822D0*TD + 0.0003036D0*TD2
     *         + 0.0018D0*DCOS( (159.D0+ 19.D0*TD)*RAD )
         EL4 =  83.353243D0   + 4069.013711D0*TD - 0.010324D0 *TD2
         EL5 = 234.955444D0   + 1934.136185D0*TD - 0.002076D0 *TD2
         EL6 = 282.937348D0      + 1.719533D0*TD + 0.0004597D0*TD2
         EL7 = 248.1D0       + 32964.47D0    *TD
         EL8 =  81.5D0       + 22518.44D0    *TD
C
C        SUM OF 1200 TIDAL WAVES 
C
         AMPX = 0.D0
         DO 1000 I = 1, MAXWV
            FACTXI = (AMPL(I) + AMPL2(I)*TD) * FACTX(I)
            TMX = ( EL1*DOOD(1,I) + EL2*DOOD(2,I) + EL3*DOOD(3,I)
     *            + EL4*DOOD(4,I) + EL5*DOOD(5,I) + EL6*DOOD(6,I)
     *            + EL7*DOOD(7,I) + EL8*DOOD(8,I) ) * RAD
     *            + PHASEI(I)
            AMPX = AMPX + FACTXI * DCOS( TMX )
 1000    CONTINUE
         XT(K) = AMPX
C
 2000 CONTINUE
C
C
      RETURN
      END
      DOUBLE PRECISION FUNCTION TJULID( IY, IM, ID, IH, MI )
C
C     CALCULATION OF JULIAN DATE
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      DIMENSION  DAYS(12)
C
      DATA   DAYS /   0.D0,  31.D0,  59.D0,  90.D0, 120.D0, 151.D0,
     *              181.D0, 212.D0, 243.D0, 273.D0, 304.D0, 334.D0 /
C
C
      IYY = MOD( IY, 1900 )
      IF( IYY.LE.0 .OR. IYY.GE.200 )  THEN
         WRITE(6,*)  'ERROR AT FUNCTION TJULID        IY =', IY
         STOP
      END IF
C
      TJULID =   DBLE( IYY*365 + ID + IYY/4 ) + DAYS(IM)
     *         + DBLE(IH)/24.D0 + DBLE(MI)/1440.D0 + 2415019.5D0
C
      IF( MOD(IYY,4).EQ.0 .AND. IM.LE.2 )  TJULID = TJULID - 1.D0
C
      RETURN
      END
